function [vModelResults,vClassifyResults,vProbabilities,vTestIndices] = crossvalidation_ny_ikfd(ny, L, v)
%CROSSVAL K-fold cross validation.

error(nargchk(3, 4, nargin))
[N] = numel(L);
L   = L(:);
if nargin < 2
  v = [];
end

if nargin < 3
  opt = [];
end

if isempty(v)
  V = 1:N;
  v = N;
elseif length(v) == 1
  if ~isa(v, 'double') | ~isreal(v) | v <= 0 | round(v) ~= v
    error('V must be a positive, non-zero integer.')
  elseif v > N
    error('V must be less than the number of observations.')
  elseif v == N
    V = 1:N;
  else
    w = v*fix(N/v);
    r = randperm(v);
    V(randperm(N)) = [reshape(repmat(1:v, w/v, 1), w, 1); ...
		      r(1:N-w)'];
  end
else
  if ~isa(v, 'double') | ~isreal(v) | prod(size(v)) ~= length(v) | ...
	any(round(v) ~= v | v <= 0 | isinf(v))
    error(['Cross-validation set indeces V must be a vector of' ...
	   ' positive, finite, non-zero integers.'])
  elseif length(v) ~= N
    error(['Length of set indeces V must be same as number of' ...
	   ' observations in X and K.'])
  elseif ~all(sum(sparse(1:N, v, 1)))
    error('Set indeces V may not have empty groups.')
  end
  
  V = v;
  v = max(V);
end
  
c = zeros(N, 1);
vModelResults    = [];
vClassifyResults = [];
vProbabilities   = [];
vTestIndices     = [];
vLandmarks       = ny{3};
for i = 1:v
  fprintf('*')
  s = warning;
  warning off
  
  % determine training model
  CurrentSet = find(V ~= i); 
  %fKernelPtrTrain = @(X,Y)fKernelPtr(CurrentSet(X),CurrentSet(Y));    
  CurrentTrainingSet = [CurrentSet(:);vLandmarks(:)];
  nyTrain{1} = ny{1}(CurrentTrainingSet,:);
  nyTrain{2} = ny{2};
  nyTrain{3} = numel(CurrentSet)+1:numel(CurrentTrainingSet);
  nyTrain{4} = ny{4}(CurrentTrainingSet);
  fKernelPtr = @(X,Y)ny{1}(X,:)*ny{2}*ny{1}(Y,:)';
  T         = cputime;
  W         = ny_ikfd(nyTrain,nyTrain{4});
  dTime     = cputime-T;
  vUnique   = W.labels;
  E         = getdata(W);
  NrClasses = numel(E);
  
  sModel = struct('Models',{W},'Labels',{vUnique},'V',{V},'T',dTime);  
  warning(s);
  
  % classify test data
  CurrentTestSet = find(V == i); 
  CurrentFullSet = [CurrentSet CurrentTestSet]; % thats only to have order train : test
  mPred          = zeros(length(CurrentTestSet),NrClasses);
  for(k=1:NrClasses)
         C=getdata(E{k});
         for(i=1:length(CurrentTestSet))
             mPred(i,k) = C.rot'*fKernelPtr(CurrentTestSet(i), CurrentTrainingSet)'+C.offset;
         end
  end
  Probabilities   = normcdf(mPred)';
  Probabilities(isnan(Probabilities)) = 0;
  PredictedLabels = argmax(Probabilities);
  dAcc = sum(vUnique(PredictedLabels)==L(CurrentTestSet))./numel(CurrentTestSet);
  vClassifyResults = [vClassifyResults dAcc];
  vProbabilities   = [vProbabilities; {Probabilities}];
  vTestIndices     = [vTestIndices; CurrentTestSet(:)];
  vModelResults    = [vModelResults sModel];  
end
