% Singular value decomposition based on a nystroem approximated matrix
% The left-singular vectors of K are eigenvectors of KK*.
% The right-singular vectors of K are eigenvectors of K*K.
% The non-zero singular values of K (found on the diagonal entries of sigma) are the square roots of the non-zero eigenvalues of both M*M and MM*.
%
% Note: 
%      1)   ny_K is here not necessarily symmetric - hence U != V
%      2)   valid rank of U,V is obviously limited to the number of non-zero singular values
% This is correct upto the sign of the singular vectors
function [U, S, V] = svd_ny(ny_K,RSample)
U = [];
V = [];
[N,m]   = size(ny_K{1});
nyKK{1} = [];
nyKK{2} = [];

% right singular vectors - V
Ckk = zeros(N,m);
K1   = ny_K{1};
Temp = (K1 * ny_K{2});
for(i=1:m)
    k        = RSample(i);
    Column   = (Temp * ny_K{3}(:,k));
    Ckk(:,i) = ((Column' * ny_K{1}) * (ny_K{2} * ny_K{3}))';
end
nyKK{1} = Ckk;
Temp    = Ckk(RSample,:);
Temp    = 0.5*(Temp+Temp');
nyKK{1}(RSample,:) = Temp; % symmetrize
nyKK{2} = pinv(Temp);
[C, A]  = eig_ny(nyKK);
S       = sqrt(A);
[Y,I]   = sort(diag(S),'descend');
V       = C(:,I);

% left singular vectors - U
% like above but ny_K{1} and ny_K{3} switched
Ckk = zeros(N,m);
Temp = ny_K{1};
ny_K{1} = ny_K{3}';
ny_K{2} = ny_K{2}';
ny_K{3} = Temp';
K1   = ny_K{1};
K3   = ny_K{3};
Temp = (K1 * ny_K{2});
for(i=1:m)
    k        = RSample(i);
    Column   = (Temp * ny_K{3}(:,k));
    Ckk(:,i) = ((Column' * ny_K{1}) * (ny_K{2} * ny_K{3}))';
end
nyKK{1} = Ckk;
Temp    = Ckk(RSample,:);
Temp    = 0.5*(Temp+Temp');
nyKK{1}(RSample,:) = Temp; % symmetrize
nyKK{2} = pinv(Temp);
[C, A]  = eig_ny(nyKK);
S       = sqrt(A);
[Y,I]   = sort(diag(S),'descend');
U       = C(:,I);

% S should be roughly the same for both ways

